@@ -62,6 +62,8 @@ pub fn run(opt: Opt) -> Result<(), Error> {
 
     // For each filepath in the input vector...
     for (num, item) in files.iter().enumerate() {
+        let start_time = Instant::now();
+
         let filepath = item;
         let mut filename = filepath.clone().to_string();
 
@@ -178,17 +180,22 @@ pub fn run(opt: Opt) -> Result<(), Error> {
         };
 
         let csum_recv = recv.delta.as_ref().map(|r| r.csum);
-        let checksum = handle.map(|s| s.join().expect("calc_file_hash panicked"));
+        let mut checksum: Option<Hash> = None;
+        if recv.ack == TeleportInitStatus::Overwrite {
+            checksum = handle.map(|s| s.join().expect("calc_file_hash panicked"));
+        }
 
         if checksum != None && checksum == csum_recv {
             // File matches hash
             send_delta_complete(stream, file)?;
         } else {
             // Send file data
-            send(stream, file, header, recv.delta)?;
+            send(stream, file, &header, recv.delta)?;
         }
 
-        println!(" done!");
+        let duration = start_time.elapsed();
+        let speed = (header.filesize as f64 * 8.0) / duration.as_secs() as f64 / 1024.0 / 1024.0;
+        println!(" done! Time: {:.2?} Speed: {:.3} Mbps", duration, speed);
     }
     Ok(())
 }
@@ -232,7 +239,7 @@ fn send_delta_complete(mut stream: TcpStream, file: File) -> Result<(), Error> {
 fn send(
     mut stream: TcpStream,
     mut file: File,
-    header: TeleportInit,
+    header: &TeleportInit,
     delta: Option<TeleportDelta>,
 ) -> Result<(), Error> {
     let mut buf = Vec::<u8>::new();
